﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace WorldMapKitLayerUtility
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void btnBrowseDllFilePath_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "OsmWorldMapKitLayer.dll | OsmWorldMapKitLayer.dll";
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                txtBoxDllFilePath.Text = dialog.FileName;
            }
        }

        private void btnGenerateCSV_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveToCsvDialog = new SaveFileDialog();
            saveToCsvDialog.Filter = "CSV Files (*.csv)|*.csv";
            saveToCsvDialog.FileName = "OsmWorldMapKitLayer Rendering Coverage Reporter.csv";
            if (saveToCsvDialog.ShowDialog() == DialogResult.OK)
            {
                string csvFilePath = saveToCsvDialog.FileName;
                string worldMapKitLayerDllFilePath = txtBoxDllFilePath.Text;

                if (Path.GetFileName(worldMapKitLayerDllFilePath).Equals("OsmWorldMapKitLayer.dll", StringComparison.InvariantCultureIgnoreCase)
                    && File.Exists(worldMapKitLayerDllFilePath) && Path.GetExtension(csvFilePath).Equals(".csv", StringComparison.InvariantCultureIgnoreCase))
                {
                    try
                    {
                        ExportFeatureLayerTable(worldMapKitLayerDllFilePath, csvFilePath);

                        MessageBox.Show("Exported to " + csvFilePath, "Success", MessageBoxButtons.OK);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK);
                    }
                }
                else
                {
                    MessageBox.Show("Invalid filepath(s)!", "Error", MessageBoxButtons.OK);
                }
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnViewRenderingCoverage_Click(object sender, EventArgs e)
        {
            // Clear the GridView to load new result.
            dataGridRenderingCoverage.Columns.Clear();
            dataGridRenderingCoverage.Rows.Clear();

            // Define new Columns
            dataGridRenderingCoverage.Columns.Add("", "");
            dataGridRenderingCoverage.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.DisplayedCells;
            for (int i = 1; i <= 20; i++)
            {
                dataGridRenderingCoverage.Columns.Add("ZL" + i, "ZL" + i);
                dataGridRenderingCoverage.Columns[i].AutoSizeMode = DataGridViewAutoSizeColumnMode.ColumnHeader;
            }

            string WorldMapKitLayerDllFilePath = txtBoxDllFilePath.Text;
            // Get the Layers included in the specified WorldMapKit layer *.DLL, and load result in GridView
            if (Path.GetFileName(WorldMapKitLayerDllFilePath).Equals("OsmWorldMapKitLayer.dll", StringComparison.InvariantCultureIgnoreCase) && File.Exists(WorldMapKitLayerDllFilePath))
            {
                Dictionary<int, Collection<FeatureLayer>> includedFeatureLayers = GetAllFeatureLayersCore(GetLayersFromAssembly(WorldMapKitLayerDllFilePath));

                if (includedFeatureLayers.Count > 0)
                {
                    dataGridRenderingCoverage.Rows.Add(includedFeatureLayers[0].Count);
                    for (int i = 0; i < includedFeatureLayers[0].Count; i++)
                    {
                        dataGridRenderingCoverage.Rows[i].Cells[0].Value = includedFeatureLayers[0][i].Name;
                    }

                    for (int i = 1; i <= 20; i++)
                    {
                        for (int j = 0; j < includedFeatureLayers[0].Count; j++)
                        {
                            if (includedFeatureLayers[i].Contains(includedFeatureLayers[0][j]))
                            {
                                dataGridRenderingCoverage.Rows[j].Cells[i].Value = "X";
                            }
                        }
                    }
                }
            }
        }

        private void ExportFeatureLayerTable(string dllFilePath, string outputCsvFilePath)
        {
            Dictionary<int, Collection<FeatureLayer>> featureLayerTable = GetAllFeatureLayersCore(GetLayersFromAssembly(dllFilePath));

            using (FileStream outputCsvStream = new FileStream(outputCsvFilePath, FileMode.Create))
            {
                StreamWriter csvfileWriter = new StreamWriter(outputCsvStream);

                //set up first line of feature layer names
                csvfileWriter.Write(",");
                for (int i = 1; i <= 20; i++)
                {
                    csvfileWriter.Write("ZL" + i + ",");
                }
                csvfileWriter.WriteLine("");

                foreach (FeatureLayer featureLayer in featureLayerTable[0])
                {
                    csvfileWriter.Write(featureLayer.Name);

                    for (int j = 1; j <= 20; j++)
                    {
                        if (featureLayerTable[j].Contains(featureLayer))
                        {
                            csvfileWriter.Write(",X");
                        }
                        else
                        {
                            csvfileWriter.Write(",");
                        }
                    }
                    csvfileWriter.WriteLine("");
                }
                csvfileWriter.Close();
            }
        }

        private Collection<FeatureLayer> GetLayersFromAssembly(string dllFilePath)
        {
            Assembly osmWorldMapKitLayerDll = Assembly.LoadFile(dllFilePath);
            Type osmWorldMapKitLayer = osmWorldMapKitLayerDll.GetType("ThinkGeo.MapSuite.Core.OsmWorldMapKitLayer");
            Type osmWorldMapKitDatabaseType = osmWorldMapKitLayerDll.GetType("ThinkGeo.MapSuite.Core.OsmWorldMapKitDatabaseType");

            object osmWorldMapKitLayerInstance = Activator.CreateInstance(osmWorldMapKitLayer, null);

            PropertyInfo connectionString = osmWorldMapKitLayer.GetProperty("ConnectionString");
            connectionString.SetValue(osmWorldMapKitLayerInstance, @"Data Source=..\..\Resource\table_source.sqlite;Version=3;Read Only=True;");

            MethodInfo methodInfo = osmWorldMapKitLayer.GetMethod("Open");
            methodInfo.Invoke(osmWorldMapKitLayerInstance, null);

            PropertyInfo layersInfo = osmWorldMapKitLayer.GetProperty("Layers");
            object osmWorldMapKitLayers = layersInfo.GetValue(osmWorldMapKitLayerInstance, null);

            return (Collection<FeatureLayer>)osmWorldMapKitLayers;
        }

        private bool CheckIfDefaultStyleSet(ZoomLevel zoomLevel)
        {
            bool defaultStyleSet = true;
            ZoomLevel defaultZoomLevel = new ZoomLevel();

            if (!(zoomLevel.DefaultAreaStyle == defaultZoomLevel.DefaultAreaStyle))
            {
                defaultStyleSet = false;
            }
            else if (!(zoomLevel.DefaultLineStyle == defaultZoomLevel.DefaultLineStyle))
            {
                defaultStyleSet = false;
            }
            else if (!(zoomLevel.DefaultPointStyle == defaultZoomLevel.DefaultPointStyle))
            {
                defaultStyleSet = false;
            }
            else if (!(zoomLevel.DefaultTextStyle == defaultZoomLevel.DefaultTextStyle))
            {
                defaultStyleSet = false;
            }

            return defaultStyleSet;
        }

        private Dictionary<int, Collection<FeatureLayer>> GetAllFeatureLayersCore(Collection<FeatureLayer> featureLayers)
        {
            Dictionary<int, Collection<FeatureLayer>> featureLayersInAllZoomLevels = new Dictionary<int, Collection<FeatureLayer>>();
            for (int i = 0; i <= 20; i++)
            {
                featureLayersInAllZoomLevels.Add(i, new Collection<FeatureLayer>());
            }

            foreach (FeatureLayer layer in featureLayers)
            {
                featureLayersInAllZoomLevels[0].Add(layer);
                Collection<ZoomLevel> layerZoomLevels = layer.ZoomLevelSet.GetZoomLevels();

                for (int i = 1; i <= 20; i++)
                {
                    ZoomLevel zoomLevel = layerZoomLevels[i - 1];

                    if (!(zoomLevel.CustomStyles.Count == 0 || CheckIfDefaultStyleSet(zoomLevel)))
                    {
                        if ((int)zoomLevel.ApplyUntilZoomLevel == 0)
                        {
                            if (!featureLayersInAllZoomLevels[i].Contains(layer))
                            {
                                featureLayersInAllZoomLevels[i].Add(layer);
                            }
                        }
                        else
                        {
                            for (int j = i; j <= (int)zoomLevel.ApplyUntilZoomLevel; j++)
                            {
                                if (!featureLayersInAllZoomLevels[j].Contains(layer))
                                {
                                    featureLayersInAllZoomLevels[j].Add(layer);
                                }
                            }
                        }
                    }
                }
            }

            return featureLayersInAllZoomLevels;
        }
    }
}
